@@ -1,7 +1,7 @@
 # rm(list = ls())
 # Sys.setenv("R_TESTS" = "")
 library(testthat)
-library(collapse)
+# library(collapse)
 
 test_check("collapse")
 
